#!/usr/bin/perl

# $Id: lipm_prepare_gaps_4circos.pl 1619 2018-08-06 11:58:20Z gouzy $

BEGIN
{
	use File::Basename;
	my $dirprg=dirname($0);
   	$ENV{PATH} = "$dirprg:$ENV{PATH}";
	$ENV{LANG}='C';
	unshift @INC, "$dirprg/../corelib";
	unshift @INC, "$dirprg/../lib";
	$VERSION = do { my @r = (q$Rev: 1619 $ =~ /\d+/g); sprintf "%d"."%02d" x $#r, @r };
}

=head1 NAME


=head1 SYNOPSIS

=head1 DESCRIPTION

Prepare tracks to visualize gaps in primary assemblies (i.e: obtained from a contiger like CANU, FALCON, smartdenovo, etc.) when compared to the finalized genome

Requirement
- a multifasta file of the genome
- at least one primary assembly copied or linked in the --assembliesdir
- minimap2 and bedtools installed in the PATH

Optional
- a multifasta file of the corrected reads

=cut

use strict;
use warnings;
use General;
use GeneralBioinfo;
use ParamParser;

my $DEF_MINCTGLEN        = 20000; # minimum length of contigs
my $DEF_MINWINSIZE       = 250000;
my $DEF_MODE             = 'minimap2';
my $DEF_MINCHRLEN4CIRCOS = 1000000;
my $DEF_CIRCOSSHAREDDIR  = "/usr/share/circos";
my $DEF_NCPUS            = 8;
my $DEF_PARAM_MINIMAP2   = "-x asm5 --min-occ-floor=200 --score-N 90 -r 90 --no-long-join "; # try not to go through ~100nt gaps.
my $DEF_PARAM_MINIMAP2_CORRREADS   = "-x asm5";
my $DEF_PARAM_BLASTN     = "-outfmt 6 -dust no -word_size 28 -perc_identity 95 -evalue 1e-200";
my $DEF_MINGAPLEN        = 0;
my $DEF_MINHSPLEN        = 0;

MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',\&Usage,'minctglen=i','circosdir=s','genome=s','workdir=s','mode=s','assembliesdir=s','correctedreads=s','force','mws4corrected=i','minchrlen4circos=i','circosshareddir=s','ncpus','mingaplen=i','minhsplen=i');

	# Init and check environnement 
	#=============================
	$o_param->AssertFileExists('genome');
	$o_param->AssertDirExists('assembliesdir');
	$o_param->AssertDefined('circosdir');
	my $circosdir = $o_param->Get('circosdir');
	$o_param->SetUnlessDefined('workdir',"$circosdir/tmpdir");
	$o_param->SetUnlessDefined('minctglen',$DEF_MINCTGLEN);
	$o_param->SetUnlessDefined('mws4corrected',$DEF_MINWINSIZE);
	$o_param->SetUnlessDefined('mode',$DEF_MODE);
	$o_param->SetUnlessDefined('minchrlen4circos',$DEF_MINCHRLEN4CIRCOS);
	$o_param->AssertAllowedValue('mode','minimap2','blastn');
	$o_param->SetUnlessDefined('circosshareddir',$DEF_CIRCOSSHAREDDIR);
	$o_param->SetUnlessDefined('ncpus',$DEF_NCPUS);
	$o_param->SetUnlessDefined('mingaplen',$DEF_MINGAPLEN);
	$o_param->SetUnlessDefined('minhsplen',$DEF_MINHSPLEN);

	my $workdir          = $o_param->Get('workdir');
	my $assembliesdir    = $o_param->Get('assembliesdir');
	my $minctglen        = $o_param->Get('minctglen');
	my $genome           = $o_param->Get('genome');
	my $force            = $o_param->IsDefined('force');
	my $mode             = $o_param->Get('mode');
	my $ncpus            = $o_param->Get('ncpus');
	my $minchrlen4circos = $o_param->Get('minchrlen4circos');
	my $mingaplen        = $o_param->Get('mingaplen');
	my $minhsplen        = $o_param->Get('minhsplen');

	foreach my $f ($circosdir,$workdir,$assembliesdir,$genome)
	{
		$f = "$ENV{PWD}/$f" if ( $f !~ /^\// );
		mkdir $f if ( ! -e $f );
	}

	my $f_log = &GetStreamOut("$circosdir/LOG");

	# test if dependencies are in PATH
	foreach my $exe ('bedtools','minimap2','gzip','sort','lipm_Nstreches_to_gff3.pl','lipm_fastaminlen.pl','lipm_fasta2seqlen.pl','lipm_blast.pl','lipm_tab_to_tab_nr.pl')
	{
		my $exepath = &GetExePath($exe);
		print $f_log "$exepath\n";
	}

	# generate the file of sequence length required by bedtools complement
	my $genomeseqlen="$workdir/genome.seqlen";
	&System("lipm_fasta2seqlen.pl --in $genome --out $genomeseqlen",$f_log,$force,$genomeseqlen);

	# Generate the tracks corresponding to gaps in primary assemblies
	#================================================================

	# build the list the list of primary assemblies available
	my @a_primaryablies = `ls $assembliesdir/*`;
	chomp(@a_primaryablies);

	if ( scalar(@a_primaryablies) < 1 )
	{
		Carp::croak("No fasta file found in --assembliesdir $assembliesdir\n");
	}

	# for each assembly, extract contigs longer than minctglen and init the list of files of filtered contigs
	my @a_ablies = ();
	foreach my $ably (@a_primaryablies)
	{
		my ($d,$f) = $ably =~ /(.+)\/(.+)/;
		my $out = "$workdir/$f.$minctglen.fa";
		&System("lipm_fastaminlen.pl --in $ably --minlen $minctglen --out $out",$f_log,$force,$out);
		push(@a_ablies,$out);
	}

	my $workdirgenome = '';
	if ( $mode eq 'blastn' )
	{
		my($dgen,$fgen) = $genome =~ /(.+)\/(.+)/;
		$workdirgenome = "$workdir/$fgen";
		&System("cd $workdir ; rm -f $workdirgenome ; ln -s $genome $workdirgenome",$f_log,$force,undef);
		&System("cd $workdir ; makeblastdb -in $workdirgenome -dbtype nucl",$f_log,$force,"$workdirgenome.nhr");
	}
	
	my @a_abliesbed=();
	my @a_segments=();
	foreach my $ably (@a_ablies)
	{
		my ($dgen,$gfile) = $genome =~ /(.+)\/(.+)/;
		my ($d,$f) = $ably =~ /(.+)\/(.+)/;
		my $bed='';
		if ( $mode eq 'minimap2' )
		{
			my $minimap2param = $DEF_PARAM_MINIMAP2;
			my $tagparam = $minimap2param;
			$tagparam =~ s/\s/_/g;
			my $paf = "$workdir/$gfile.vs.$f.minimap2.$tagparam.tp:A:P.paf.gz";
			&System("minimap2 $minimap2param $genome $ably | grep tp:A:P | gzip -c  > $paf",$f_log,$force,$paf);
			$bed = "$paf.bed";
			&PAFToBed($paf,$bed,$f_log,$force,$bed);
		}
		elsif ( $mode eq 'blastn')
		{
			my $img_param = $DEF_PARAM_BLASTN;
			my $tagparam = $img_param;
			$tagparam =~ s/\s/_/g;
			my $blastm8 =  "$workdir/$gfile.vs.$f.blastn.$tagparam.m8";
			&System("cd $workdir ; lipm_blast.pl --ncpus $ncpus --nt_per_slice 5000000 blastn $img_param -query $ably -out $blastm8 -db $workdirgenome ",$f_log,$force,$blastm8);
			my $blastm8nr1 = "$blastm8.0.95.nr1";
			&System("cd $workdir ; lipm_tab_to_tab_nr.pl --in  $blastm8 --out $blastm8nr1 --pcs 0.95",$f_log,$force,$blastm8nr1);
			$bed = "$blastm8nr1.bed";
			&M8ToBed($blastm8nr1,$bed,$f_log,$force,$bed);
		}
		my $filtlenbed = "$circosdir/$f.minhsplen$minhsplen.bed";
		&RemoveShortHSPs($bed,$filtlenbed,$minhsplen,$f_log,$force,$filtlenbed);
		push(@a_abliesbed,$filtlenbed);

		my $complbed = "$circosdir/$f.minhsplen$minhsplen.complement.4circos.bed";
		&System("bedtools complement -g $genomeseqlen -i $filtlenbed > $complbed",$f_log,$force,$complbed);
		my $complmingaplenbed = "$circosdir/$f.minhsplen$minhsplen.complement.mingaplen$mingaplen.4circos.bed";
		&RemoveShortGaps($complbed,$complmingaplenbed,$mingaplen,$f_log,$force,$complmingaplenbed);

		push(@a_segments,$complmingaplenbed);
	}

	# Generate the track corresponding to gaps in all assemblies
	#===========================================================

	my $allbeds = join(' ',@a_abliesbed);
	my $allbed = "$workdir/allbeds.bed";
	&System("cat $allbeds | sort -k 2,2n | sort -k 1,1 -s > $allbed",$f_log,$force,$allbed);
	my $complallbed = "$allbed.complement.4circos.bed";
	&System("bedtools complement -g $genomeseqlen -i $allbed > $complallbed",$f_log,$force,$complallbed);
	#my $complallmingaplenbed = "$allbed.complement.mingaplen$mingaplen.4circos.bed";
	#&RemoveShortGaps($complallbed,$complallmingaplenbed,$mingaplen,$f_log,$force,$complallmingaplenbed);
	#push(@a_segments,$complallmingaplenbed); # this track aims to be compared to the gap in the final assembly (via the detection of N streches => next track)
											  # In many projects, few Ns are added between contigs during the scaffolding
											  # So, if the filter is applied on the gap size, it becomes confusing to see no gap in all assemblies (this track) 
											  # and a gap in the final assembly (next track)
											  # conversely, without filtering on the gap size, gaps can appear in this consensus track without gap (above mingaplen)
											  # in the assembly vs genome tracks
											  # best consensus ==> no filter based on the gap size for this track

	push(@a_segments,$complallbed); 

	# Generate the track corresponding to gaps in the genome
	#=======================================================

	&System("lipm_Nstreches_to_gff3.pl --infile $genome --outfile $workdir/genome.N.gff3 --minlen 1",$f_log,$force,"$workdir/genome.N.gff3",$f_log,$force,"$workdir/genome.N.gff3");
	&Gff3ToBed("$workdir/genome.N.gff3","$circosdir/genome.N.4circos.bed",$f_log,$force,"$circosdir/genome.N.4circos.bed");

	# Generate the track corresponding to the coverage of corrected long reads
	#=========================================================================

	my $corr4circos='';
	if ( $o_param->IsDefined('correctedreads') )
	{
		my $correctedreads= $o_param->Get('correctedreads');
		$correctedreads = "$ENV{PWD}/$correctedreads" if ( $correctedreads !~ /^\// );
		if ( ! -e $correctedreads )
		{
			Carp::croak("No file found --correctedreads $correctedreads\n");
		}
		my ($dgen,$gfile) = $genome =~ /(.+)\/(.+)/;
		my ($dcorr,$cfile) = $correctedreads =~ /(.+)\/(.+)/;
		my $paf = "$workdir/$gfile.vs.$cfile.tp:A:P.paf.gz";

		&System("minimap2  $DEF_PARAM_MINIMAP2_CORRREADS $genome $correctedreads | grep tp:A:P | gzip -c > $paf",$f_log,$force,$paf);
		my $corrbed = "$workdir/$gfile.vs.$cfile.tp:A:P.bed";

		&PAFToBed($paf,$corrbed,$f_log,$force,$corrbed);

		my $corrcov = "$workdir/$gfile.vs.$cfile.tp:A:P.bed.genomecov.bed.gz";
		&System("bedtools genomecov -d -i $corrbed -g $genomeseqlen | gzip -c > $corrcov",$f_log,$force,$corrcov); # slow for large genome

		my $winsize = $o_param->Get('mws4corrected');
		$corr4circos = "$circosdir/correctedreads.mappingcoverage.winsize$winsize.4circos";
		&ComputeMeanCoveragePerWin($corrcov,$corr4circos,$winsize,$f_log,$force,$corr4circos);
	}

	# Dump the circos file
	#=========================================================================

	my $circosfile = "$circosdir/circos.conf";
	&CircosHead($circosfile,$genomeseqlen,$minchrlen4circos);
	&CircosLine($circosfile,"<plots>");
	my $max = 0.99;
	my $step= 0.04;
	my $delta= 0.01;
	foreach my $file (@a_segments)
	{
		&CircosSegment($circosfile,$max,$max-$step,$file,"blue");
		$max=($max-$step-$delta);
	}
	&CircosSegment($circosfile,$max,$max-$step,"$circosdir/genome.N.4circos.bed","black");
	$max=($max-$step-$delta);

	if ( $o_param->IsDefined('correctedreads') )
	{
		&CircosScatter($circosfile,$max,$max-2*$step,$corr4circos);
	}
	&CircosLine($circosfile,"</plots>");
	&CircosTail("$circosfile",$o_param->Get('circosshareddir'));

	$f_log->close;
	print "Configuration file for circos (see http://circos.ca for tuning): $circosfile\n";
	print "(edit/adapt/check the path of <<include.. in the $circosfile file before running circos --conf $circosfile --outputfile myplot)\n";
	print "Log file: $circosdir/LOG\n";
}


sub ComputeMeanCoveragePerWin
{
my($in,$out,$winsize,$f_log,$force,$successfile) = @_;

	print $f_log "#sub ComputeMeanCoveragePerWin($in,$out,$winsize) ";
	if ( ! $force && -e $successfile && ! -z $successfile )
	{
		print $f_log ": already done, skip\n";
		return;
	}
	my $f_in  = &GetStreamIn($in);
	my $f_out = &GetStreamOut($out);
	my $tot=0;
	my $min=0;
	my $max=0;
	my $prev='';
	my($chr,$pos,$count);
	while(my $l=<$f_in>)
	{
		chomp($l);
		($chr,$pos,$count)=split(' ',$l);
		if ( $tot && ($chr ne $prev || $pos % $winsize == 0 ) )
		{
				my $delta = ($max-$min);
				printf $f_out "$prev\t".($min-1)."\t$max\t%.2f\n",$tot/$delta if ($delta > 0 );
				$tot=0;
		}
		$min = $pos if ( $tot == 0 );
		$max = $pos;
		$tot+=$count;
		$prev = $chr;
	}
	my $delta = ($max-$min);
	printf $f_out "$chr\t".($min-1)."\t$max\t%.2f\n",$tot/($max-$min) if ($delta > 0 );
	print $f_log ": done\n";
	$f_in->close;
	$f_out->close;
}

# filter : Bed => Bed

sub RemoveShortGaps
{
my($in,$out,$mingaplen,$f_log,$force,$successfile)=@_;

	print $f_log "#sub RemoveShortGaps($in,$out,$mingaplen) // filter";
	if ( ! $force && -e $successfile && ! -z $successfile )
	{
		print $f_log ": already done, skip\n";
		return;
	}
	&RemoveShorts($in,$out,$mingaplen);
	print $f_log ": done\n";
}

# filter : Bed => Bed

sub RemoveShortHSPs
{
my($in,$out,$minhsplen,$f_log,$force,$successfile)=@_;

	print $f_log "#sub RemoveShortHSPs($in,$out,$minhsplen) // filter";
	if ( ! $force && -e $successfile && ! -z $successfile )
	{
		print $f_log ": already done, skip\n";
		return;
	}
	&RemoveShorts($in,$out,$minhsplen);
	print $f_log ": done\n";
}

sub RemoveShorts
{
my($in,$out,$cutoff)=@_;

	my $f_in  = &GetStreamIn($in);
	my $f_out = &GetStreamOut($out);
	my %h_res=();
	while(my $l=<$f_in>)
	{
		chomp($l);
		my @a_ = split(/\t/,$l);
		next if ( $a_[2]-$a_[1] < $cutoff );
		print $f_out "$l\n";
	}
	$f_in->close;
	$f_out->close;
}

# format converter: GFF3 => Bed

sub Gff3ToBed
{
my($in,$out,$f_log,$force,$successfile)=@_;

	print $f_log "#sub Gff3ToBed($in,$out) // format conversion";
	if ( ! $force && -e $successfile && ! -z $successfile )
	{
		print $f_log ": already done, skip\n";
		return;
	}
	my $f_in  = &GetStreamIn($in);
	my $f_out = &GetStreamOut($out);
	my %h_res=();
	while(my $l=<$f_in>)
	{
		chomp($l);
		&ParseGFF3Line(\$l,1,\%h_res);
		print $f_out "$h_res{seqid}\t".($h_res{start}-1)."\t$h_res{end}\n";
	}
	print $f_log ": done\n";
	$f_in->close;
	$f_out->close;
}

# format converter: M8 => Bed

sub M8ToBed
{
my($in,$out,$f_log,$force,$successfile)=@_;

	print $f_log "#sub M8ToBed($in,$out) // format conversion";
	if ( ! $force && -e $successfile && ! -z $successfile )
	{
		print $f_log ": already done, skip\n";
		return;
	}
	my $f_in  = &GetStreamIn($in);
	my $f_out = &GetStreamOut("| sort -k 2,2n | sort -k 1,1 -s > $out");
	my %h_res=();
	while(my $l=<$f_in>)
	{
		chomp($l);
		&ParseM8Line(\$l,2,\%h_res);
		print $f_out "$h_res{query}\t".($h_res{qstart}-1)."\t$h_res{qend}\n";
	}
	print $f_log ": done\n";
	$f_in->close;
	$f_out->close;
}

# format converter: Paf(minimap) => Bed

sub PAFToBed
{
my($in,$out,$f_log,$force,$successfile)=@_;

	print $f_log "#sub PAFToBed($in,$out) // format conversion ";
	if ( ! $force && -e $successfile && ! -z $successfile )
	{
		print $f_log ": already done, skip\n";
		return;
	}
	my $f_in  = &GetStreamIn($in);
	my $f_out = &GetStreamOut("| sort -k 2,2n | sort -k 1,1 -s > $out");
	my %h_res=();
#m150131_041601_44444_c100782302550000001823155108051591_s1_p0/1/0_4522  4522    0       4522    -       RchiOBHmChr7    70290800        62442504        62447032        4200    4532    60      tp:A:P  cm:i:420        s1:i:4200       s2:i:292        dv:f:0.0039
	while(my $l=<$f_in>)
	{
		chomp($l);
		my @a_ = split(/\t/,$l);
		next if ( $#a_ < 8 );
		my ($dv) = $a_[$#a_] =~ /dv:f:(\S+)/;
		my ($pci) = (defined($dv)) ? sprintf("%.2f",100-$dv) : 0;
		print $f_out "$a_[5]\t$a_[7]\t$a_[8]\t$pci\n";
	}
	print $f_log ": done\n";
	$f_in->close;
	$f_out->close;
}


sub CircosHead
{
my($out,$genomeseqlen,$minseqlen)=@_;

	my($first,$last)=&BuildKaryotype($genomeseqlen,"$out.karyotype",$minseqlen);

	my $f_out = &GetStreamOut($out);

	select($f_out);

print<<END
karyotype = $out.karyotype
chromosomes_units = 1000000
chromosomes_display_default = yes

<ideogram>
	<spacing>
		default = 0.005r
		<pairwise $last $first>
			spacing = 10r 
		</pairwise>
	</spacing>

	radius    = 0.9r
	thickness = 20p
	fill      = yes
	stroke_color     = black
	stroke_thickness = 1p
	show_label       = yes
	label_font       = default 
	label_radius     = 1r + 75p
	label_size       = 30
	label_parallel   = yes
</ideogram>
END
;
	select(STDOUT);
	$f_out->close;
}

sub CircosSegment
{
my($out,$max,$min,$file,$color)=@_;

	my $f_out = new IO::File(">>$out");
	select($f_out);

print<<END
<plot>
type = highlight
r1 = ${max}r
r0 = ${min}r
color = $color
fill_color = $color
file = $file
</plot>
END
;
	select(STDOUT);
	$f_out->close;
}

sub CircosScatter
{
my($out,$max,$min,$file)=@_;

	my $f_out = new IO::File(">>$out");
	select($f_out);

print<<END
<plot>
show  = yes
type  = scatter
file  = $file
r1    = ${max}r
r0    = ${min}r
max   = 100
min   = 0
glyph            = circle
glyph_size       = 4
color            = blue
stroke_color     = dblue
stroke_thickness = 1
<backgrounds>
	<background>
		color = vvlgrey
	</background>
</backgrounds>
<axes>
	<axis>
	color     = dgrey
	thickness = 1
	spacing   = 0.25r
	</axis>
</axes>
</plot>
END
;
	select(STDOUT);
	$f_out->close;

}

sub CircosLine
{
my($out,$line)=@_;

	my $f_out = new IO::File(">>$out");
	print $f_out "$line\n";
	$f_out->close;
}

sub CircosTail
{
my($out,$confdir)=@_;

	my $f_out = new IO::File(">>$out");

	select($f_out);

print<<END
<image>
	<<include $confdir/etc/image.conf>>
</image>

<<include $confdir/etc/colors_fonts_patterns.conf>>
<<include $confdir/etc/housekeeping.conf>>

# ticks.conf
show_ticks          = yes
show_tick_labels    = yes

<ticks>
	radius           = 1r
	color            = black
	thickness        = 2p
	multiplier       = 1e-6
	format           = %.0f

	<tick>
		spacing        = 10u
		size           = 10p
		show_label     = yes
		label_size     = 15p
		label_offset   = 10p
		format         = %.0f
	</tick>
</ticks>
END
;
	select(STDOUT);
	$f_out->close;

}

#=====================================
# Build the circos configuration files
#=====================================

sub BuildKaryotype
{
my($genomeseqlen,$karyo,$minseqlen)=@_;
	
	my $first=undef;
	my $last =undef;
	my $f_in=&GetStreamIn($genomeseqlen);
	my $f_out=&GetStreamOut($karyo);
	while(my $l=<$f_in>)
	{
		chomp($l);
		my ($chr,$len)=split("\t",$l);
		if ( $len >= $minseqlen )
		{
			$first = $chr if ( !defined($first) );
			$last  = $chr;
			my $chrdisp  = $chr;
			$chrdisp =~ s/.+(chr.+)/$1/i;
			print $f_out "chr\t-\t$chr\t$chrdisp\t0\t$len\tgrey\n";
		}
	}
	$f_in->close;
	$f_out->close;
	return($first,$last);
}

sub System
{
my($cde,$f_log,$force,$successfile)=@_;

	print $f_log "$cde";
	if ( ! $force && defined($successfile) && -e $successfile && ! -z $successfile )
	{
		print $f_log ": already done, skip\n";
		return;
	}
	my $retcode = system($cde);
	if( $retcode != 0 || -e "core" )
	{
		die "FATAL ERROR running command >$cde<\n";
	}
	print $f_log ": done\n";
}

sub Usage
{
print STDERR<<END
This script aims to : 
- prepare tabulated files to visualize gaps in primary assemblies (i.e: obtained with contigers like CANU, FALCON, smartdenovo, etc.)
when compared to the finalized genome. 
- generate a circos configuration file that uses the tabulated files as tracks.

$0
	--help          this message
Mandatory
	--circosdir          dirname          directory for output files
	--genome             fastafile        genome sequence in fasta format
	--assembliesdir      dirname          directory for raw assemblies in fasta format (Warning: only fasta files in this directory)
Optional
	--minctglen          integer          [$DEF_MINCTGLEN] minimun length of contigs in assemblies
	--mingaplen          integer          [$DEF_MINGAPLEN] minimun length of the gaps 
	                                      a short gap can be due to a sequence divergence between the consensus of the two sequences but
	                                      not to an actual gap in the primary assembly
	--minhsplen          integer          [$DEF_MINHSPLEN] minimun length of the HSPs
	--minchrlen4circos   integer          [$DEF_MINCHRLEN4CIRCOS] mininum length of genome sequence used for the circos plot
	--correctedreads     fastafile        corrected reads (obtained either by CANU or FALCON and mapped with minimap2 -x asm5)
	--mws4corrected      integer          [$DEF_MINWINSIZE] length of the windows for computing the mean of corrected reads coverage
	--workdir            dirname          [circosdir/tmpdir] directory where are stored temporary files
	--force                               force computations (by default, computations are not performed when expected output file is found and non empty)
	--mode               minimap2|blastn  [$DEF_MODE] select the software for assembly vs genome comparisons
	--circosshareddir    dirname          [$DEF_CIRCOSSHAREDDIR] path of the directory containing shared configuration files of circos
	--ncpus              integer          [$DEF_NCPUS] number of cpus used for blastn 

Process overview of the gap detection:
--mode minimap2 
	- minimap2 (https://github.com/lh3/minimap2) is used for sequence comparisons ($DEF_PARAM_MINIMAP2)
	- only primary hits are considered (grep tp:A:P)
	- paf output files are converted in bed format
	Note: minimap2 generates spurious short primary alignments that can conduct to an underestimation of the number of gaps in primary alignments.
	The use of --minhsplen can decrease the rate of false negative gaps. 
	Tip: a value of several thousands can be used (e.g 2000, two versions of the same genome are compared + minimap2 is very sensitive => large hits are expected)
--mode blastn
	- blastn (ftp://ftp.ncbi.nlm.nih.gov/blast/executables/blast+/LATEST/) is used for assemblies vs genome comparisons ($DEF_PARAM_BLASTN) 
	         and minimap2 for correctedreads vs genome comparison ($DEF_PARAM_MINIMAP2_CORRREADS)
	- the script lipm_tab_to_tab_nr.pl (based on bedtools) is used to remove secondary hits
	- blast m8 output files are converted in bed format
	Note: the blastn method overestimates the number of gaps (e.g: when non polisheds contig are used, the similarity can locally decrease a 
	      lot and several non overlaping HSPs can be generated for a given region). The use of --mingaplen to remove short gaps can decrease
		  the rate of false positive gaps. Warning: do not use a value higher than the length of gaps in the pseudomolecules
- then the command 'bedtools complement' is used to identify regions not spanned by hits (https://github.com/arq5x/bedtools2/releases)
- gaps shorted than --mingaplen (default=$DEF_MINGAPLEN) will be removed

The circos file (--circosdir/circos.conf) will contain
- 1 track for gaps in each of the fasta files found in --assembliesdir (sorted with the ls command of your shell)
- 1 track for gaps in all assemblies (Note: --mingaplen not used to generate this track)
- 1 track for gaps in the genome sequence (N streches in --genome)
- 1 optional track for the plot of the coverage of correctedreads (if --correctereads is provided; can take hours)

END
}

