#
# $Id: LipmSingleton.pm 855 2012-03-13 13:35:30Z manu $
#

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

 LipmSingleton - Deriving from LipmObject, defines a singleton

=head1 DESCRIPTION

 Define an object deriving from LipmSingleton to get a singleton behaviour for your object:
 
 Inside MySingleton.pm:
    use base qw( LipmSingleton );
    Write your _Init as usual
    
 Inside the file using the singleton:
    my $o_my_singleton = New MySingleton( my_parameters );
    
    The sub New defined here will be called, if the singleton is not already created the _Init SUB is called, else the already existing object is returned
    
    
  If your singleton has data to initialize from an external file, you may call _InitModuleData from the _Init sub
  
=head1 EXAMPLE

 See the objects GenbankTags.pm, SeqOntology.pm, folder lib/BioFileConverter

=cut

package LipmSingleton;

use strict;
use warnings;
use General;
use Carp;
use base qw( LipmObject );
use 5.10.0;
use FindBin;

=head2 LipmSingleton::New

 Title    : New
 Usage    : The constructor
 Access   : Public
 Function : 
 The internal hash ($self) is created and blessed using a state variable
            The protected function _Init is then called, letting the derived classes
            to perform their own initialization
 Args     : the class, followed by object-specific arguments
 Returns  : The object
 Globals  : none

 Note     : The object will exist whatever happen inside _Init.
            In consequence, you can have an object with no additionnal attribute.

 Attributes :
    __o_logger : Logger object.

=cut

sub LipmSingleton::New
{
	# Warning, the arguments are used only the FIRST TIME the singleton is created !
	my ( $class, @args ) = @_;
	
	# Will be created only the first time, thanks to the keyword state
	state %h_self_container;
	
	if ( !exists $h_self_container{$class} )
	{
		my $self = bless( {}, $class );
		$self->_Set( '__o_logger', undef );
		
		# Useful for _InitModuleData;
		$self->_Set('__InitModuleDataAlreadyCalled',&FALSE);

		$self->_Init( @args );
		$h_self_container{$class} = $self;
	}
	
	return $h_self_container{$class};
}


=head2 LipmSingleton::Clone

 Title    : Clone
 Usage    : Clone (=copy) the object
 Access   : Public
 Function : Removed from LipmObject (meaning we are doing incorrrect inheritance, sorry)
 Args     : none (-- or just the object)
 Returns  : The new object
 Globals  : none

=cut
sub Clone
{
    croak "ERROR - Please do not clone a singleton ";
}


=head2 Procedure _Init

 Title    : _Init
 Usage    : $self->SUPER::_Init( -o_logger => $o_logger );
            $self->SUPER::_Init( $o_logger );
 Access   : Protected
 Procedure: if called with 1 param, set it to __o_logger
            if called with an even number of params, set the -o_logger to __o_logger
            Call SetLogger
 Args     : $o_logger, a Logger object, facultative
 Globals  : none

=cut
sub _Init
{
	my $self   = shift;
	my @a_args = @_;

	if   ( 1 == @a_args )
    {
        $self->SetLogger( $a_args[0] );
    }
    elsif( 0 == @a_args % 2 )
	{
		my %h_args = @a_args;
		$self->SetLogger( $h_args{ -o_logger } );
	}
}

=head2 Function InitModuleData

 Title    : InitModuleData
 Usage    : eval $self -> _InitModuleData ('some_file_to_init.pl','other_file_to_init.pl'); if $@ die $@;
 Function : Search the files in the @INC directories
            Those files should contain some code initializing global variables (with our)
            Read the files and return the concatenated files as a string, it should be used with an eval from the calling module
            Mark the initialization as done.
 Args     : The file name, the file should live in a directory from @INC, preferably the same directory as the module
			The first found file is the only used
 WARNING  : there may be security concern with the method as the file may be written by the user (if the directory . is in @INC)
 Return   : The concatenated files, this is probably some code perl you should execute with eval

=cut

sub _InitModuleData
{
	my $self = shift;
	my (@a_data_file) = @_;
	my $data ='';

	if ( $self->_Get('__InitModuleDataAlreadyCalled') != &TRUE)
	{
		foreach my $data_file (@a_data_file)
		{
			my $found = &FALSE;
			foreach my $d (@INC)
			{
				my $f = File::Spec->catfile($d,$data_file);
				#print "$f\n";
				if ( -e $f )
				{
					my $fh_f = GetStreamIn($f);
					my @a_data = <$fh_f>;
					chomp @a_data;
					$data .= join "\n",@a_data;
					$found = &TRUE;
					last;
					#print "===================== \n";
				}
			}
			if ($found==&FALSE)
			{
				die "ERROR - Cannot load data file $data_file - \@INC=( " . join ( ',',@INC ) . " )";
			}
		}
		$self->_Set('__InitModuleDataAlreadyCalled',&TRUE);
	}
	return $data;
}

1;

__END__

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

